<?php

/***************************
* PonVFTP                  *
* Written By: Pongles      *
* Website: pongles.com     *
* Licence: GPL             *
****************************/

// purpose: ensure that the user has access to the module
// returns: bool
function CanAccessModule( $iID )
{
    global $g_rgModules, $g_rgUser;
    
    $rgModule = $g_rgModules[ $iID ];
    
    if( !empty( $rgModule[ "function" ] ) )
    {
        if( !call_user_func( $rgModule[ "function" ] ) ) {
            return false;
		}
    }
    else if( !CheckAccess( $rgModule[ "access" ], $g_rgUser[ "access" ] ) )
    {
        return false;
    }
    
    return true;
}

// purpose: gets the default module
// returns: none
function ModDefaultModule( &$strDefaultModule )
{
    global $g_bAnonShowBrowseFirst, $g_bAllowAnon;

    // needs to be a guest etc
    if( !$g_bAnonShowBrowseFirst || !$g_bAllowAnon || GetUserAccess( ) != ACCESS_GUEST ) {
        return;
	}
    
    // browse module!
    $strDefaultModule = MODULE_BROWSE;
}

// purpose: loads the module and handles logout
// returns: array
function LoadModule()
{
	global $g_rgModules;

	$strModule = $_GET[ "mod" ];

	if( $strModule == "logout" && CheckUserAccess( ACCESS_USER ) )
	{
		ClearCookies();

		SendLogoutHTML();
		exit;
	}
    
    if( $strModule == "download" && CheckUserAccess( ACCESS_GUEST ) )
    {
        require( "download.php" );
        exit;
    }                     

	foreach( $g_rgModules as $iModuleID => $rgModule )
    {
		if( $rgModule[ "link" ] != $strModule ) {
            continue;
		}
            
        if( !CanAccessModule( $iModuleID ) ) {
            continue;
		}
            
        return $g_rgModules[ $iModuleID ];
	}
    
    // set default module
    $strDefaultModule = MODULE_DEFAULT;
    
    // try and change
    ModDefaultModule( $strDefaultModule );
    
    // return module data
    return $g_rgModules[ $strDefaultModule ];
}

// purpose: get module name
// returns: string
function GetModulePath( $strExtension = "" )
{
    global $g_rgModData;
    return "modules/" . ( ( !empty( $strExtension ) ) ? $strExtension . "_" : "" ) . $g_rgModData[ "link" ] . ".php";
}

// purpose: sets up the user and handles login
// returns: array
function SetupUser( )
{
	global $rgUsers;
	
	if( $_COOKIE[ "username" ] )
	{
		if( FindUser( $_COOKIE[ "username" ], $rgUser ) && 
                $_COOKIE[ "passhash" ] == $rgUser[ "passhash" ] )
        {
		    return $rgUser;
		}
		
		ClearCookies();
		SendHeader();
	}
	else if( $_POST[ "login" ] )
	{
		if( ValidateUser( $_POST[ "username" ], $_POST[ "password" ], $strMD5Pass ) )
        {
			setcookie( "username", $_POST[ "username" ] );
			setcookie( "passhash", $strMD5Pass );

			SendLoginHTML();
			exit;
		}
		else
		{
			SendHeader("index.php?page=login&failed=1");
		}
	}

	return array( "name" => "Guest", "access" => ACCESS_GUEST );
}

// purpose: finds a user in the database
// returns: bool
function FindUser( $strUsername, &$rgUser )
{
	global $g_rgUsers;
	
	for( $i = 0; $i < sizeof( $g_rgUsers ); $i++)
	{
		$rgFindUser = $g_rgUsers[ $i ];
		
		if( $strUsername != $rgFindUser[ "username" ] ) {
			continue;
		}
			
		$rgUser = $rgFindUser;
        return true;
	}
	
	return false;
}

// purpose: finds a user in the database through their ID
// returns: bool
function FindUserID( $iID, &$rgDeleteUser )
{
	global $g_rgUsers;
    
    if( !IsValidUserID( $iID ) ) {
        return false;
	}
        
   $rgDeleteUser = $g_rgUsers[ $iID ];
   return true;
}

// purpose: checks for valid userid
// returns: bool
function IsValidUserID( $iUserID )
{
    global $g_rgUsers;
    return( $iUserID >= 0 && $iUserID < sizeof( $g_rgUsers ) );
}

// purpose: finds a user in the database
// returns: bool
function LoadUser( $iUserID, &$rgUser )
{
    global $g_rgUsers;
    
    if( !IsValidUserID( $iUserID ) ) {
        return false;
	}
    
    $rgUser = $g_rgUsers[ $iUserID ];
    return true;
}

// purpose: updates a user
// returns: bool
function UpdateUser( $iUserID, $rgUser )
{
    global $g_rgUsers;
    
    if( !IsValidUserID( $iUserID ) ) {
        return false;
	}
    
    $g_rgUsers[ $iUserID ] = $rgUser;
    return true;    
}

// purpose: creates a user
// returns: int
function CreateUser( )
{
    global $g_rgUsers;
    
    $g_rgUsers[] = array( );
    
    return sizeof( $g_rgUsers ) - 1;
}

// purpose: removes a user
// returns: none
function RemoveUser( $iUserID )
{
    global $g_rgUsers;
    unset( $g_rgUsers[ $iUserID ] );
}

// purpose: validates access
// returns bool
function IsValidAccessID( $iAccessID )
{
    return ( $iAccessID >= ACCESS_GUEST && $iAccessID <= ACCESS_SADMIN );
}

// purpose: validates user
// returns: bool
function ValidateUser( $strUsername, $strPassword, &$strMD5Pass )
{
    if( !FindUser( $strUsername, $rgUser ) ) {
        return false;
	}
	
	if( empty( $rgUser[ "passhash" ] ) ) {
		return true;
	}
        
    $strMD5Pass = CreatePassHash( $strPassword, $rgUser[ "passsalt" ] );

    return ( $rgUser[ "passhash" ] == $strMD5Pass );
}

// purpose: create password
// returns: string
function CreatePassHash( $strPassword, $strSalt )
{
	return CreateFinalPassHash( md5( $strPassword ), $strSalt );
}

// purpose: create password
// returns: string
function CreateFinalPassHash( $strMD5Password, $strSalt )
{
	return md5( $strMD5Password . $strSalt );
}
// purpose: clears the cookies
// returns: none
function ClearCookies()
{
	setcookie( "username", "" );
	setcookie( "passhash", "" );
}

// purpose: wrapper for header
// returns: none
function SendHeader( $strLocation = "index.php" )
{
	header( "Location: ".$strLocation );
}

// purpose:
// returns: bool
function CheckAccess( $iAccessRequired, $iAccess )
{
	return $iAccess >= $iAccessRequired;
}

// purpose:
// returns: bool
function CheckUserAccess( $iAccessRequired )
{
	global $g_rgUser;
    return CheckAccess( $iAccessRequired, $g_rgUser[ "access" ] );
}

// purpose
// returns: bool
function GetUserAccess( )
{
	global $g_rgUser;
    return $g_rgUser[ "access" ];
}

// purpose:
// returns: bool
function IsUserAdmin( )
{
    return ( CheckUserAccess( ACCESS_SADMIN ) || CheckUserAccess( ACCESS_ADMIN ) );
}

// purpose: print out of the links
// returns: none
function PrintLinks()
{
	global $g_rgModules;
	
	foreach( $g_rgModules as $iModuleID => $rgModule )
    {
        if( !CanAccessModule( $iModuleID ) ) {
            continue;
		}
        
		$strLinks .= " - <a href=\"index.php?mod=". $rgModule[ "link" ] ."\">" . $rgModule[ "name" ] . "</a>";
	}

	echo $strLinks . " -";
}

// purpose: remove crazy stuff from path strings
// returns: string
function ParsePath( $strPath )
{
    // replace HTML trickery
    $strPath = str_replace( "%92", "\\", $strPath );
    $strPath = str_replace( "%47", "/", $strPath );
    $strPath = str_replace( "%46", ".", $strPath );

    // need to replace "//" and remove "../"    
	$strPath = str_replace( "\\", "/", $strPath );
    $strPath = str_replace( "../", "", $strPath );
    
    // split it up and put it back together
    if( !empty( $strPath ) )
    {
        $rgPath = explode( "/", $strPath );
        
        foreach( $rgPath as $strSegPath )
        {
            if( empty( $strSegPath ) ) {
                continue;
			}
            
            $rgFinalPath[] = $strSegPath;
        }
    }
    
    if( !empty( $rgFinalPath ) ) {
        $strPath = implode( "/", $rgFinalPath );
	}
    else {
        $strPath = "";
	}
    
    return $strPath;
}

// purpose: finds paths
// returns: bool (but it does write to $strPath and $strFullPath etc)
function FindPaths( $strRawPath, &$strPath, &$strFullPath, &$strJail, &$strURLPath )
{
	global $g_rgUser, $g_strFileUpload, $g_bUserJail, $g_strAnonJail;
    
    $strPath = $strURLPath = ParsePath( $strRawPath ); 
    
    // construct full path
	$strFullPath = $g_strFileUpload;
    
    // add in extra slashes if needed
    if( !empty( $strPath ) && substr( $strPath, 0, 1 ) != "/" ) {
        $strPath = "/" . $strPath;
	}
     
    if( substr( $strPath, 0, -1 ) != "/" ) {
         $strPath .= "/";
	}
 
	// work out user jails
	if( !IsUserAdmin( ) )
	{
        if( GetUserAccess() == ACCESS_GUEST ) {
            $strJail = $g_strAnonJail;
		}
		else if( $g_bUserJail && GetUserAccess() == ACCESS_USER ) {
			$strJail = $g_rgUser[ "home" ];
		}
            
        if( !empty( $strJail ) ) {
            $strFullPath = $strFullPath . "/" . $strJail;
		}
	}
    
    // finish full path
    $strFullPath .= $strPath;

	// ensure its not empty and ensure its a directory
	return ( !empty( $strFullPath ) && @is_dir( $strFullPath ) );
}

// purpose: can upload a file at a path
// returns: bool
function CanUploadFile( $strPath )
{
    global $g_rgUser, $g_bUserJail;
    
	// admins can always upload
	if( IsUserAdmin() ) {
		return true;
	}
        
	// check permissions for anon
	if( GetUserAccess() == ACCESS_GUEST && !$g_bAnonUpload ) {
		return false;
	}
        
    // ensure that they're not readonly
    if( $g_rgUser[ "readonly" ] ) {
        return false;
	}
        
    // ensure that their inside their jail
    $rgHome = explode( "/", $g_rgUser[ "home" ] );
    $rgPath = explode( "/", $strPath );
    
    if( empty( $rgHome ) ) {
        return true;
	}
        
    if( $g_bUserJail ) {
        array_shift( $rgHome );
	}
        
    array_shift( $rgPath );
    
    foreach( $rgHome as $iID => $strPathSeg )
    {
        if( empty( $strPathSeg ) ) {
            continue;
		}
            
        if( $rgPath[ $iID ] != $rgHome[ $iID ] ) {
            return false;
		}
    }
    
    return true;
}

// purpose: wether or not a user can manage folders (assumes that the user can upload)
// returns: bool
function CanManageFolders( )
{
    global $g_bUserFolders;
    
     if( IsUserAdmin( ) ) {
        return true;
	}
        
     return $g_bUserFolders;
}

// purpose: wether or not a user can manage folders
// returns: bool
function CanManageFolders2( $strPath )
{
    if( !CanUploadFile( $strPath ) ) {
        return false;
	}

     return CanManageFolders( );    
}

// purpose: work out the previous directory
// returns: string (bool if error)
function FindBackPath( $strPath )
{
	global $g_strFileUpload;
	
	$strFullPath = $g_strFileUpload . $strPath;
	
	if( !is_dir( $strFullPath ) || !is_readable( $strFullPath ) ) {
		return false;
	}
		
	$strPath = trim( $strPath, "/" );
		 
	$rgPreviousPath = explode( "/", $strPath );
	
	if( sizeof( $rgPreviousPath ) != 0 )
	{
		array_pop($rgPreviousPath);
		
		return implode( "/", $rgPreviousPath );
	}
	
	return "";
}

// purpose: produce a human readable filesize
// returns: string
function GetHumanFileSize( $strFilePath, $iSize )
{
	$rgMag = array("", "KB", "MB", "GB", "TB", "PB");

    $iPos = 0;
    
    while( $iSize >= 1024 )
    {
		$iSize /= 1024;
		$iPos++;
	}

	return round( $iSize, 2 )." ".$rgMag[ $iPos ];
}

// purpose: finds the file type string
// returns: bool
function FindTypeType2( $strPath, &$strFileSuffix )
{
    // find filename
	$strFileName = basename( $strPath );
	
	if( empty( $strFileName ) ) {
		return false;
	}

    // work out extension
	$rgFileName = explode( ".", $strFileName );	
	$strFileSuffix = $rgFileName[ 1 ];
	
	return ( !empty( $strFileSuffix ) );
}

// purpose: finds the file type
// returns: none (string and path to image)
function FindFileType( $strPath, &$strFileType, &$strImagePath )
{
	global $g_rgExtensions;
    
    // set default stuff
    $strFileType = "Unknown File";
    $strImagePath = "000";
	
    // find filename
    if( !FindTypeType2( $strPath, $strFileSuffix ) ) {
        return;
	}
    
    // find it	
	foreach( $g_rgExtensions as $rgExtension )
	{
        if( !is_array( $rgExtension[ "ext" ] ) ) {
            continue;
		}
            
		foreach( $rgExtension[ "ext" ] as $iExtensionSuffixID => $strExtensionSuffix )
		{
			if( $strFileSuffix != $strExtensionSuffix ) {
				continue;
			}
			
			$strFileType = $rgExtension[ "name" ];
			$strImagePath = $rgExtension[ "image" ];
			
			return;
		}
	}
}

// purpose: send a error message
// returns: none
function SendError( $strError )
{
	echo "<div class=\"error\"><b>Error:</b> " . $strError . "</div><br>";
}


// purpose: sends the login HTML
// returns: none
function SendLoginHTML()
{
?>
<html>
<head>
<title>Please Stand By</title>
<meta http-equiv="refresh" content="2; url=index.php">
<style>
a:link, a:visited, a:active { text-decoration: underline; color: #FFFFFF; }
a:hover { text-decoration: none; }
TABLE, TR, TD { font-family: Verdana, Tahoma, Arial; font-size: 9pt; color: #FFFFFF; }
.white { background-color: #FFFFFF; }
.row1 { background-color: #676767; color: #FFFFFF; font-family: Arial, Helvetica, sans-serif; }
</style>
</head>
<body bgcolor="#FFFFFF">
<table width="95%" height="85%" align="center">
<tr>
<td valign="middle">
<table align="center" border="0" cellspacing="1" cellpadding="0" bgcolor="#A8C5E1">
<tbody><tr> 
<td class="white">
<table width="100%" border="0" cellspacing="1" cellpadding="12">
<tr> 
<td width="100%" align="center" class="row1">
Thanks, you are now logged in<br><br>
Please wait while we transfer you <a href="index.php">here</a><br>
</td>
</tr>
</table>
</td>
</tr>
</table>
</td>
</tr>
</table>
</body>
</html>
<?php
}

// purpose: sends the logout HTML
// returns: none
function SendLogoutHTML()
{
?>
<html>
<head>
<title>Please Stand By</title>
<meta http-equiv="refresh" content="2; url=index.php">
<style>
a:link, a:visited, a:active { text-decoration: underline; color: #FFFFFF; }
a:hover { text-decoration: none; }
TABLE, TR, TD { font-family: Verdana, Tahoma, Arial; font-size: 9pt; color: #FFFFFF; }
.white { background-color: #FFFFFF; }
.row1 { background-color: #676767; color: #FFFFFF; font-family: Arial, Helvetica, sans-serif; }
</style>
</head>
<body bgcolor="#FFFFFF">
<table width="95%" height="85%" align="center">
<tr>
<td valign="middle">
<table align="center" border="0" cellspacing="1" cellpadding="0" bgcolor="#A8C5E1">
<tbody><tr> 
<td class="white">
<table width="100%" border="0" cellspacing="1" cellpadding="12">
<tr> 

<td width="100%" align="center" class="row1">
Thanks, You are now logged out<br><br>
Please wait while we transfer you <a href="index.php">here</a><br>
</td>
</tr>
</table>
</td>
</tr>
</table>
</td>
</tr>
</table>
</body>
</html>
<?php	
}

// purpose: starts, ends and displays page creation time
// author: ed [at] twixcoding [dot] com
class CTimer
{
   function getmicrotime()
   {
       list($usec, $sec) = explode(" ", microtime());
       return ((float)$usec + (float)$sec);
   }   
  
   function starttime()
   {
   $this->st = $this->getmicrotime();
   }
      
   function displaytime()
   {
       $this->et = $this->getmicrotime();
       return round(($this->et - $this->st), 3);
   }
}

// purpose: validates an email address
// returns: bool
function IsValidEmail( $strEmail )
{
	return eregi( "^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $strEmail );
}

// purpose: rewrites vdata
// returns: bool
function RewriteVData( )
{
    global $g_rgUsers;
    
    $hFile = @fopen( "vdata.php", "w" );
    
    if( !$hFile ) {
        return false;
	}
	
    $strDate = date( "l dS \of F Y h:i:s A" );
    
    // write users
    fwrite( $hFile, "<?php\n\n// generated file, do not edit (".$strDate.")!!!\n\n");
    
    foreach( $g_rgUsers as $iUserID => $rgUser )
    {
        fwrite( $hFile, "\$g_rgUsers[] = array( " );
        
        foreach( $rgUser as $strData => $strValue )
        {
            if( $strData == "id" ) {
                continue;
            }
                      
            fwrite( $hFile, "\"" . $strData . "\" => \"" . $strValue . "\", " );
        }
        
        fwrite( $hFile, "\"id\" => \"" . $iUserID . "\" );\n" ); 
    }
    
    fwrite( $hFile, "\n\n?>" );
    
    fclose( $hFile );
    
    return true;
}

// purpose: create random string
// returns: string
function CreateRandom( $iLength )
{
	$strPool = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	$iLastChar = strlen( $strPool ) - 1;

	for( $i = 0; $i < $iLength; $i++ ) {
		$strRandom .= $strPool[ mt_rand( 0, $iLastChar ) ];
	}

	return $strRandom;
}

// purpose: create pass salt
// returns: string
function CreatePassSalt( )
{
    return CreateRandom( 4 );
}

// purpose: sends emails
class CMail
{
   var $strTo;
   var $strFrom;
   var $strSubject;
   var $strMessage;
  
   function send()
   {
       return mail( $this->$strTo, $this->$strSubject, $this->$strMessage, "From: ".$this->strFrom."\r\n" );
   }
}

// purpose: determins wether or not we're using email for passwords
// returns: bool
function IsSendingPass( )
{
    return ( !empty( $g_strMailServerName ) && IsValidEmail( $g_strMailServerEmail ) );
}

// purpose: checks for valid filename
// returns: bool
function IsValidPathSeg( $strFilename )
{
    return !ereg( "\\\\|/", $strFilename );
}

// purpose: same as strrpos, except $needle can be a string
// returns: string (or false if not found)
// author: gordon at kanazawa-gu dot ac dot jp
function strrpos_string($haystack, $needle, $offset=0)
{
   $strrpos = false;
   if (is_string($haystack) && is_string($needle) && is_numeric($offset))
   {
       $strlen = strlen($haystack);
       $strpos = strpos(strrev(substr($haystack, $offset)), strrev($needle));
       if (is_numeric($strpos)) {
           $strrpos = $strlen - $strpos - strlen($needle);
       }
   }
   return $strrpos;
}

// purpose: removes the last element from a path
// returns: bool
function RemoveLastPathSeg( &$strPath, $strBaseName )
{
    $iFoundPos = strrpos_string( $strPath, $strBaseName );
    
    if( $iFoundPos === false ) {
        return;
	}
    
    $strPath = substr( $strPath, 0, strlen( $strPath ) - strlen( substr( $strPath, $iFoundPos ) ) );
}

// purpose: gets the edit path
// returns: bool
function FindEditPath( &$strPath, &$strFullPath, &$strEditName, &$strURLPath )
{
    // check path and put together
    if( !FindPaths( $_GET[ "path" ], $strPath, $strFullPath, $strJail, $strURLPath ) ) {
        return false;
	}
        
    // get the name of what they're wanting to edit and fiddle it into the path 
    $strFileName = $_GET[ "file" ];
    
    if( empty( $strFileName ) )
    {
        $bIsFile = false;
        $strEditName = basename( $strPath );
    }
    else
    {
        $bIsFile = true;
        
        if( !IsValidPathSeg( $strFileName ) ) {
            return false;
		}
        
        $strEditName = $strFileName;
    }
       
    if( empty( $strEditName ) ) {
        return false;
	}
        
    // do a dirty hack to remove the $strEditName out of the path's
    if( !$bIsFile )
    {
        RemoveLastPathSeg( $strPath, $strEditName );
        RemoveLastPathSeg( $strFullPath, $strEditName );
    }
        
    // ensure that its valid 
    if( $bIsFile && !@is_file( $strFullPath . $strFileName ) ) {
        return false;
	}
    
    // work out URL path
    $strURLPath = "&path=" . $strPath;
    
    if( $bIsFile ) {
        $strURLPath .= "&file=". $strFileName;
	}
    else {
        $strURLPath .= $strEditName;
	}
    
    return true;    
}

// purpose: parse home dir
// returns: string
function ParseHomeDir( $strHome )
{
    $strHome = ParsePath( $strHome );
    $strHome = trim( $strHome, "/" );
    
    return $strHome;
}

// purpose:
// returns: bool
function IsReadOnly( )
{
    global $g_rgUser;
    return ( $g_rgUser[ "access" ] == ACCESS_USER && $g_rgUser[ "readonly"] );
}

// purpose: defines scandir for non-PHP5 users
// author: www.mdsjack.bo.it
function php4_scandir($dir, $sort = 0)
{
	if (PHP_VERSION >= '5') {
        return scandir($dir, $sort);
	}
  
	$dirmap = array();
  
	if(!is_dir($dir)) {
       return false;
	}
  
	$dir = opendir($dir);
  
	while (false !== ($file = readdir($dir))) {
       $dirmap[] = $file;
	}
  
   closedir($dir);
  
   ($sort == 1) ? rsort($dirmap) : sort($dirmap);
  
   return $dirmap;
}

// purpose: recurivly remove folders and files
// returns: bool
// author: brianleeholub at yahoo dot com (and Pongles)
function RemoveDirectory( $dir )
{
    $dir_contents = php4_scandir( $dir );

    foreach( $dir_contents as $item )
    {
        if( $item == "." || $item == ".." ) {
            continue;
		}

		$path = $dir . "/" . $item;
            
        if( is_dir( $path ) )
        {
            if( !RemoveDirectory( $path ) ) {
                return false;
			}
        }
        else if( file_exists( $path ) )
        {
            if( !unlink( $path ) ) {
                return false;
			}
        }
    }
       
    return rmdir( $dir );
}

// purpose:
// returns: bool
function IsValidSortCode( $iSortCode )
{
    return ( $iSortCode >= 0 && $iSortCode <= LAST_SORT );
}

?>